/*
 * BeanShell.java - Bean shell
 * Copyright (C) 1999, 2000 Slava Pestov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import bsh.*;
import console.Console;
import console.Shell;
import java.io.File;
import org.gjt.sp.jedit.*;
import org.gjt.sp.util.Log;

public class BeanShell extends Shell
{
	public BeanShell()
	{
		super("BeanShell");
	}

	public void printInfoMessage(Console console)
	{
		console.printInfo(jEdit.getProperty("beanshell.info"));
	}

	public void execute(View view, String command, Console console)
	{
		interp.setVariable("view",view);
		interp.setVariable("console",console);

		try
		{
			interp.eval(command);
			exitCode = true;
		}
		catch(Throwable t)
		{
			Log.log(Log.ERROR,this,t);
			if(console != null)
				console.printError(t.toString());
			else
			{
				String[] args = { t.toString() };
				GUIUtilities.error(view,"bsh-error",args);
			}
			exitCode = false;
		}

		interp.setVariable("view",null);
		interp.setVariable("console",null);
	}

	public void source(View view, String file, Console console)
	{
		interp.setVariable("view",view);
		interp.setVariable("console",console);

		Buffer buffer = jEdit.getBuffer(file);

		try
		{
			if(buffer != null)
				interp.eval(buffer.getText(0,buffer.getLength()));
			else
				interp.source(file);
			exitCode = true;
		}
		catch(Throwable t)
		{
			Log.log(Log.ERROR,this,t);
			if(console != null)
				console.printError(t.toString());
			else
			{
				String[] args = { t.toString() };
				GUIUtilities.error(view,"bsh-error",args);
			}
			exitCode = false;
		}

		interp.setVariable("view",null);
		interp.setVariable("console",null);
	}

	public void stop() {}

	public boolean waitFor()
	{
		return exitCode;
	}

	public Interpreter getInterpreter()
	{
		return interp;
	}

	public String getName()
	{
		return "BeanShell";
	}

	// package-private members
	void init()
	{
		interp = new Interpreter();
		interp.setVariable("shell",this);

		Log.log(Log.MESSAGE,this,"Loading jedit.bsh from JAR file");

		try
		{
			interp.eval(getClass().getResourceAsStream("/jedit.bsh"));
		}
		catch(Throwable t)
		{
			Log.log(Log.ERROR,this,"Error loading jedit.bsh from JAR file:");
			Log.log(Log.ERROR,this,t);
		}

		File beanShellDirectory = new File(jEdit.getSettingsDirectory(),"beanshell");
		if(!beanShellDirectory.exists())
			beanShellDirectory.mkdir();
		String[] files = beanShellDirectory.list();

		if(files != null)
		{
			for(int i = 0; i < files.length; i++)
			{
				if(!files[i].toLowerCase().endsWith(".bsh"))
					continue;

				String path = MiscUtilities.constructPath(
					beanShellDirectory.getPath(),files[i]);
				Log.log(Log.MESSAGE,this,"Loading " + path);
				source(null,path,null);
			}
		}
	}

	Object method(NameSpace namespace, String name)
	{
		Object method = (Object)namespace.getMethod(name);
		if(method == null)
			return null;

		try
		{
			return interp.eval(name + "()",namespace);
		}
		catch(EvalError e)
		{
			Log.log(Log.ERROR,this,e);
			String[] args = { e.toString() };
			GUIUtilities.error(null,"bsh-error",args);
		}

		return null;
	}

	Object method(NameSpace namespace, String name, Object[] args)
	{
		Object method = (Object)namespace.getMethod(name);
		if(method == null)
			return null;

		interp.setVariable("__glue_args",args);

		try
		{
			StringBuffer buf = new StringBuffer(name);
			buf.append("(");
			for(int i = 0; i < args.length; i++)
			{
				if(i != 0)
					buf.append(",");
				buf.append("__glue_args[");
				buf.append(i);
				buf.append("]");
			}
			buf.append(")");
			return interp.eval(buf.toString(),namespace);
		}
		catch(EvalError e)
		{
			Log.log(Log.ERROR,this,e);
			String[] _args = { e.toString() };
			GUIUtilities.error(null,"bsh-error",_args);
		}

		interp.setVariable("__glue_args",null);

		return null;
	}

	// private members
	private Interpreter interp;
	private boolean exitCode;
}
